#' @keywords internal
"_PACKAGE"
## usethis namespace: start
#' @importFrom checkmate assert
#' @importFrom checkmate assert_character
#' @importFrom checkmate assert_choice
#' @importFrom checkmate assert_class
#' @importFrom checkmate assert_data_frame
#' @importFrom checkmate assert_date
#' @importFrom checkmate assert_directory
#' @importFrom checkmate assert_directory_exists
#' @importFrom checkmate assert_environment
#' @importFrom checkmate assert_integerish
#' @importFrom checkmate assert_list
#' @importFrom checkmate assert_logical
#' @importFrom checkmate assert_names
#' @importFrom checkmate assert_path_for_output
#' @importFrom checkmate assert_posixct
#' @importFrom checkmate assert_r6
#' @importFrom checkmate assert_set_equal
#' @importFrom checkmate assert_string
#' @importFrom checkmate assert_tibble
#' @importFrom checkmate assert_true
#' @importFrom checkmate check_directory_exists
#' @importFrom checkmate check_set_equal
#' @importFrom checkmate check_true
#' @importFrom checkmate expect_character
#' @importFrom checkmate expect_class
#' @importFrom checkmate expect_data_frame
#' @importFrom checkmate expect_directory_exists
#' @importFrom checkmate expect_file_exists
#' @importFrom checkmate expect_list
#' @importFrom checkmate expect_logical
#' @importFrom checkmate expect_r6
#' @importFrom checkmate expect_scalar_na
#' @importFrom checkmate test_character
#' @importFrom checkmate test_choice
#' @importFrom checkmate test_class
#' @importFrom checkmate test_directory_exists
#' @importFrom checkmate test_file_exists
#' @importFrom checkmate test_integerish
#' @importFrom checkmate test_logical
#' @importFrom checkmate test_scalar_na
#' @importFrom cli cli_abort
#' @importFrom cli cli_alert_danger
#' @importFrom cli cli_alert_info
#' @importFrom cli cli_alert_success
#' @importFrom cli cli_alert_warning
#' @importFrom cli cli_bullets
#' @importFrom cli cli_code
#' @importFrom cli cli_li
#' @importFrom cli cli_progress_bar
#' @importFrom cli cli_progress_update
#' @importFrom cli cli_text
#' @importFrom cli cli_h1
#' @importFrom cli cli_h2
#' @importFrom cli cli_h3
#' @importFrom cli cli_warn
#' @importFrom cli format_message
#' @importFrom dplyr as_tibble
#' @importFrom dplyr bind_rows
#' @importFrom dplyr filter
#' @importFrom dplyr first
#' @importFrom dplyr last
#' @importFrom dplyr pull
#' @importFrom dplyr rename
#' @importFrom dplyr select
#' @importFrom dplyr starts_with
#' @importFrom dplyr tibble
#' @importFrom R6 R6Class
#' @importFrom redcapAPI exportLogging
#' @importFrom redcapAPI exportVersion
#' @importFrom redcapAPI REDCAP_METADATA_FIELDTYPE
#' @importFrom redcapAPI REDCAP_METADATA_VALIDATION_TYPE
#' @importFrom redcapAPI REDCAP_PROJECT_PURPOSE
#' @importFrom redcapAPI REDCAP_SYSTEM_FIELDS
#' @importFrom redcapAPI redcapConnection
#' @importFrom REDCapR redcap_delete
#' @importFrom REDCapR redcap_file_download_oneshot
#' @importFrom REDCapR redcap_read
#' @importFrom REDCapR redcap_read_oneshot
#' @importFrom REDCapR redcap_report
#' @importFrom REDCapR redcap_write
#' @importFrom REDCapR sanitize_token
#' @importFrom stringr str_extract
#' @importFrom stringr str_length
#' @importFrom stringr str_match
#' @importFrom stringr str_remove
#' @importFrom stringr str_replace_all
#' @importFrom stringr str_split_fixed
#' @importFrom stringr str_to_title
#' @importFrom stringr str_trunc
#' @importFrom utils browseURL
#' @importFrom utils head
#' @importFrom utils object.size
#' @importFrom utils write.csv
## usethis namespace: end
NULL
