% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/REDCapSync-package.R
\docType{package}
\name{REDCapSync-package}
\alias{REDCapSync}
\alias{REDCapSync-package}
\title{REDCapSync: Encapsulated 'REDCap' Projects for Synchronized Data Pipelines}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Wraps dozens of 'REDCap' API endpoints into a standardized R6 object. Research Electronic Data Capture ('REDCap') is a survey and database web application software maintained by Vanderbilt University. It has a robust application programming interface (API) utilized by several R packages. 'REDCapSync' uses 'redcapAPI' and 'REDCapR' behind-the-scenes to retrieve all metadata, data, and log details for a project. To minimize unnecessary server calls, the interim 'REDCap' log is analyzed and used to only update necessary records. Furthermore, the user can define custom datasets that save to a directory. Those datasets continue to refresh when projects are synced. Having a secure, standardized, API-efficient, project-agnostic R object for 'REDCap' projects, streamlines downstream use in scripts, functions, and shiny applications.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/thecodingdocs/REDCapSync}
  \item \url{https://thecodingdocs.github.io/REDCapSync/}
  \item Report bugs at \url{https://github.com/thecodingdocs/REDCapSync/issues}
}

}
\author{
\strong{Maintainer}: Brandon Rose \email{thecodingdocs@gmail.com} (\href{https://orcid.org/0009-0009-7813-1960}{ORCID}) [copyright holder]

Authors:
\itemize{
  \item Brandon Rose \email{thecodingdocs@gmail.com} (\href{https://orcid.org/0009-0009-7813-1960}{ORCID}) [copyright holder]
}

Other contributors:
\itemize{
  \item Natalie Goulett [contributor]
}

}
\keyword{internal}
